package de.jottyfan.minecraft.quickiefabric.items;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.jottyfan.minecraft.quickiefabric.container.BackpackInventory;
import de.jottyfan.minecraft.quickiefabric.container.BackpackScreenHandler;
import de.jottyfan.minecraft.quickiefabric.init.RegistryManager;
import io.netty.buffer.Unpooled;
import net.fabricmc.fabric.api.screenhandler.v1.ExtendedScreenHandlerFactory;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.block.entity.ChestBlockEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemUsageContext;
import net.minecraft.network.PacketByteBuf;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.LiteralText;
import net.minecraft.text.Text;
import net.minecraft.text.TranslatableText;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Hand;
import net.minecraft.util.TypedActionResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

/**
 * 
 * @author jotty
 *
 */
public class ItemBackpack extends Item {
	public final static Integer SLOTSIZE = 54;

	public ItemBackpack() {
		super(new Item.Settings().group(RegistryManager.QUICKIEFABRIC_GROUP).maxCount(1));
	}

	@Override
	public TypedActionResult<ItemStack> use(World world, PlayerEntity player, Hand hand) {
		if (player.isSneaking()) {
			player.setStackInHand(hand, sortBackpackContent(player.getStackInHand(hand)));
		}
		if (!world.isClient) {
			final ItemStack itemStack = player.getStackInHand(hand);
			player.openHandledScreen(new ExtendedScreenHandlerFactory() {
				@Override
				public void writeScreenOpeningData(ServerPlayerEntity serverPlayerEntity, PacketByteBuf packetByteBuf) {
					packetByteBuf.writeItemStack(itemStack);
					packetByteBuf.writeByte(hand == Hand.MAIN_HAND ? 1 : 0);
				}

				@Override
				public Text getDisplayName() {
					return new LiteralText(RegistryManager.BACKPACK_IDENTIFIER.toString());
				}

				@Override
				public ScreenHandler createMenu(int syncId, PlayerInventory inv, PlayerEntity player) {
					PacketByteBuf buf = new PacketByteBuf(Unpooled.buffer());
					buf.writeItemStack(itemStack);
					buf.writeByte(hand == Hand.MAIN_HAND ? 1 : 0);
					return new BackpackScreenHandler(syncId, inv, buf);
				}
			});
		}
		return new TypedActionResult<ItemStack>(ActionResult.SUCCESS, player.getStackInHand(hand));
	}

	/**
	 * sort the content of the backpack; for spreaded items, combine them until the itemstack size is reached
	 * 
	 * @param itemStack the itemStack of the backpack
	 * @return the itemStack of the sorted backpack
	 */
	private ItemStack sortBackpackContent(ItemStack itemStack) {
		if (!itemStack.isEmpty()) {
			List<ItemStack> stacks = BackpackInventory.getItemsFromBackpack(itemStack);
			Map<String, List<ItemStack>> map = new HashMap<>();
			for (ItemStack stack : stacks) {
				String id = stack.getItem().getTranslationKey();
				List<ItemStack> foundStacks = map.get(id);
				if (foundStacks == null) {
					foundStacks = new ArrayList<>();
					map.put(id, foundStacks);					
				}
				foundStacks.add(stack);
			}
			BackpackInventory.setItemsToBackpack(itemStack, map.values());
		}
		return itemStack;
	}

	@Override
	public ActionResult useOnBlock(ItemUsageContext context) {
		BlockPos pos = context.getBlockPos();
		World world = context.getWorld();
		PlayerEntity player = context.getPlayer();
		BlockState blockState = world.getBlockState(pos);
		Block block = blockState.getBlock();
		if (Blocks.CHEST.equals(block)) {
			BlockEntity entity = world.getBlockEntity(pos);
			BlockEntityType<?> type = entity.getType();
			if (BlockEntityType.CHEST.equals(type)) {
				ChestBlockEntity cbe = (ChestBlockEntity) entity;
				BackpackInventory bi = new BackpackInventory(context.getStack());
				bi.setHand(context.getHand());
				if (cbe.isEmpty()) {
					for (int slot = 0; slot < ItemBackpack.SLOTSIZE; slot++) {
						if (slot < cbe.size()) {
							cbe.setStack(slot, bi.getStack(slot));
							bi.removeStack(slot);
						}
					}
					bi.onClose(player);
					if (!world.isClient) {
						player.sendMessage(new TranslatableText("msg.backpack.transfer.cleared"), false);
					}
				} else if (bi.isEmpty()) {
					for (int slot = 0; slot < ItemBackpack.SLOTSIZE; slot++) {
						if (slot < cbe.size()) {
							bi.setStack(slot, cbe.getStack(slot));
							cbe.setStack(slot, ItemStack.EMPTY);
						}
					}
					bi.onClose(player);
					if (!world.isClient) {
						player.sendMessage(new TranslatableText("msg.backpack.transfer.filled"), false);
					}
				} else {
					if (!world.isClient) {
						player.sendMessage(new TranslatableText("msg.backpack.transfer.cancel"), false);
					}
				}
			}
		}
		return super.useOnBlock(context);
	}
}
